@@ -168,12 +168,10 @@
 /*	configuration file. Setting \fBvar_idle_limit\fR to zero
 /*	disables the idle limit.
 /* DIAGNOSTICS
-/*	Problems and transactions are logged to \fBsyslogd\fR(8)
-/*	or \fBpostlogd\fR(8).
+/*	Problems and transactions are logged to \fBsyslogd\fR(8).
 /* SEE ALSO
 /*	master(8), master process
-/*	postlogd(8), Postfix logging
-/*	syslogd(8), system logging
+/*	syslogd(8) system logging
 /* LICENSE
 /* .ad
 /* .fi
@@ -197,6 +195,7 @@
 #include <sys/time.h>			/* select() */
 #include <unistd.h>
 #include <signal.h>
+#include <syslog.h>
 #include <stdlib.h>
 #include <limits.h>
 #include <string.h>
@@ -215,6 +214,7 @@
 /* Utility library. */
 
 #include <msg.h>
+#include <msg_syslog.h>
 #include <msg_vstream.h>
 #include <chroot_uid.h>
 #include <listen.h>
@@ -244,7 +244,6 @@
 #include <mail_flow.h>
 #include <mail_version.h>
 #include <bounce.h>
-#include <maillog_client.h>
 
 /* Process manager. */
 
@@ -585,6 +584,7 @@ NORETURN event_server_main(int argc, char **argv, MULTI_SERVER_FN service,...)
     const char *err;
     char   *generation;
     int     msg_vstream_needed = 0;
+    int     redo_syslog_init = 0;
     const char *dsn_filter_title;
     const char **dsn_filter_maps;
     int     retire_me_from_flags = 0;
@@ -620,7 +620,7 @@ NORETURN event_server_main(int argc, char **argv, MULTI_SERVER_FN service,...)
      * Initialize logging and exit handler. Do the syslog first, so that its
      * initialization completes before we enter the optional chroot jail.
      */
-    maillog_client_init(mail_task(var_procname), MAILLOG_CLIENT_FLAG_NONE);
+    msg_syslog_init(mail_task(var_procname), LOG_PID, LOG_FACILITY);
     if (msg_verbose)
 	msg_info("daemon started");
 
@@ -674,6 +674,8 @@ NORETURN event_server_main(int argc, char **argv, MULTI_SERVER_FN service,...)
 	    if ((err = split_nameval(oname_val, &oname, &oval)) != 0)
 		msg_fatal("invalid \"-o %s\" option value: %s", optarg, err);
 	    mail_conf_update(oname, oval);
+	    if (strcmp(oname, VAR_SYSLOG_NAME) == 0)
+		redo_syslog_init = 1;
 	    myfree(oname_val);
 	    break;
 	case 'r':
@@ -711,11 +713,11 @@ NORETURN event_server_main(int argc, char **argv, MULTI_SERVER_FN service,...)
     set_mail_conf_str(VAR_SERVNAME, service_name);
 
     /*
-     * Initialize generic parameters and re-initialize logging in case of a
-     * non-default program name or logging destination.
+     * Initialize generic parameters.
      */
     mail_params_init();
-    maillog_client_init(mail_task(var_procname), MAILLOG_CLIENT_FLAG_NONE);
+    if (redo_syslog_init)
+	msg_syslog_init(mail_task(var_procname), LOG_PID, LOG_FACILITY);
 
     /*
      * Register higher-level dictionaries and initialize the support for
