@@ -121,16 +121,22 @@
 /* .IP "CA_MAIL_SERVER_BOUNCE_INIT(const char *, const char **)"
 /*	Initialize the DSN filter for the bounce/defer service
 /*	clients with the specified map source and map names.
+/* .IP "CA_MAIL_SERVER_RETIRE_ME"
+/*	Prevent a process from being reused indefinitely. After
+/*	(var_max_use * var_max_idle) seconds or some sane constant,
+/*	terminate voluntarily when the process becomes idle.
 /* .PP
-/*	The var_use_limit variable limits the number of clients that
-/*	a server can service before it commits suicide.
-/*	This value is taken from the global \fBmain.cf\fR configuration
-/*	file. Setting \fBvar_idle_limit\fR to zero disables the client limit.
+/*	The var_use_limit variable limits the number of clients
+/*	that a server can service before it commits suicide. This
+/*	value is taken from the global \fBmain.cf\fR configuration
+/*	file. Setting \fBvar_use_limit\fR to zero disables the
+/*	client limit.
 /*
 /*	The var_idle_limit variable limits the time that a service
 /*	receives no client connection requests before it commits suicide.
+/*	Do not change this setting before calling single_server_main().
 /*	This value is taken from the global \fBmain.cf\fR configuration
-/*	file. Setting \fBvar_use_limit\fR to zero disables the idle limit.
+/*	file. Setting \fBvar_idle_limit\fR to zero disables the idle limit.
 /* DIAGNOSTICS
 /*	Problems and transactions are logged to \fBsyslogd\fR(8).
 /* BUGS
@@ -236,6 +242,15 @@ static NORETURN single_server_exit(void)
     exit(0);
 }
 
+/* single_server_retire - retire when idle */
+
+static NORETURN single_server_retire(int unused_event, void *unused_context)
+{
+    if (msg_verbose)
+	msg_info("time to retire -- exiting");
+    single_server_exit();
+}
+
 /* single_server_abort - terminate after abnormal master exit */
 
 static void single_server_abort(int unused_event, void *unused_context)
@@ -441,6 +456,8 @@ NORETURN single_server_main(int argc, char **argv, SINGLE_SERVER_FN service,...)
     int     redo_syslog_init = 0;
     const char *dsn_filter_title;
     const char **dsn_filter_maps;
+    int     retire_me_from_flags = 0;
+    int     retire_me = 0;
 
     /*
      * Process environment options as early as we can.
@@ -498,7 +515,7 @@ NORETURN single_server_main(int argc, char **argv, SINGLE_SERVER_FN service,...)
      * stderr, because no-one is going to see them.
      */
     opterr = 0;
-    while ((c = GETOPT(argc, argv, "cdDi:lm:n:o:s:St:uvVz")) > 0) {
+    while ((c = GETOPT(argc, argv, "cdDi:lm:n:o:r:s:St:uvVz")) > 0) {
 	switch (c) {
 	case 'c':
 	    root_dir = "setme";
@@ -530,6 +547,10 @@ NORETURN single_server_main(int argc, char **argv, SINGLE_SERVER_FN service,...)
 		redo_syslog_init = 1;
 	    myfree(oname_val);
 	    break;
+	case 'r':
+	    if ((retire_me_from_flags = atoi(optarg)) <= 0)
+		msg_fatal("invalid retirement time: %s", optarg);
+	    break;
 	case 's':
 	    if ((socket_count = atoi(optarg)) <= 0)
 		msg_fatal("invalid socket_count: %s", optarg);
@@ -649,6 +670,15 @@ NORETURN single_server_main(int argc, char **argv, SINGLE_SERVER_FN service,...)
 	    dsn_filter_maps = va_arg(ap, const char **);
 	    bounce_client_init(dsn_filter_title, *dsn_filter_maps);
 	    break;
+	case MAIL_SERVER_RETIRE_ME:
+	    if (retire_me_from_flags > 0)
+		retire_me = retire_me_from_flags;
+	    else if (var_idle_limit == 0 || var_use_limit == 0
+		     || var_idle_limit > 18000 / var_use_limit)
+		retire_me = 18000;
+	    else
+		retire_me = var_idle_limit * var_use_limit;
+	    break;
 	default:
 	    msg_panic("%s: unknown argument type: %d", myname, key);
 	}
@@ -765,6 +795,8 @@ NORETURN single_server_main(int argc, char **argv, SINGLE_SERVER_FN service,...)
      */
     if (var_idle_limit > 0)
 	event_request_timer(single_server_timeout, (void *) 0, var_idle_limit);
+    if (retire_me)
+	event_request_timer(single_server_retire, (void *) 0, retire_me);
     for (fd = MASTER_LISTEN_FD; fd < MASTER_LISTEN_FD + socket_count; fd++) {
 	event_enable_read(fd, single_server_accept, CAST_INT_TO_VOID_PTR(fd));
 	close_on_exec(fd, CLOSE_ON_EXEC);
