@@ -137,8 +137,8 @@
 /* STANDARDS
 /*	RFC 7159 (JSON notation)
 /* DIAGNOSTICS
-/*	Problems are logged to \fBsyslogd\fR(8) or \fBpostlogd\fR(8),
-/*	and to the standard error stream.
+/*	Problems are logged to \fBsyslogd\fR(8) and to the standard error
+/*	stream.
 /* ENVIRONMENT
 /* .ad
 /* .fi
@@ -201,8 +201,6 @@
 /*	flush(8), fast flush service
 /*	sendmail(1), Sendmail-compatible user interface
 /*	postsuper(1), privileged queue operations
-/*	postlogd(8), Postfix logging
-/*	syslogd(8), system logging
 /* README FILES
 /* .ad
 /* .fi
@@ -249,6 +247,7 @@
 #include <clean_env.h>
 #include <vstream.h>
 #include <msg_vstream.h>
+#include <msg_syslog.h>
 #include <argv.h>
 #include <safe.h>
 #include <connect.h>
@@ -273,7 +272,6 @@
 #include <valid_mailhost_addr.h>
 #include <mail_dict.h>
 #include <mail_parm_split.h>
-#include <maillog_client.h>
 
 /* Application-specific. */
 
@@ -568,14 +566,15 @@ int     main(int argc, char **argv)
 	    msg_fatal_status(EX_UNAVAILABLE, "open /dev/null: %m");
 
     /*
-     * Initialize. Set up logging. Read the global configuration file after
-     * parsing command-line arguments. Censor the process name: it is
-     * provided by the user.
+     * Initialize. Set up logging, read the global configuration file and
+     * extract configuration information. Set up signal handlers so that we
+     * can clean up incomplete output.
+     * Censor the process name: it is provided by the user.
      */
     argv[0] = "postqueue";
     msg_vstream_init(argv[0], VSTREAM_ERR);
     msg_cleanup(unavailable);
-    maillog_client_init(mail_task("postqueue"), MAILLOG_CLIENT_FLAG_NONE);
+    msg_syslog_init(mail_task("postqueue"), LOG_PID, LOG_FACILITY);
     set_mail_conf_str(VAR_PROCNAME, var_procname = mystrdup(argv[0]));
 
     /*
@@ -638,7 +637,7 @@ int     main(int argc, char **argv)
      */
     mail_conf_read();
     /* Re-evaluate mail_task() after reading main.cf. */
-    maillog_client_init(mail_task("postqueue"), MAILLOG_CLIENT_FLAG_NONE);
+    msg_syslog_init(mail_task("postqueue"), LOG_PID, LOG_FACILITY);
     mail_dict_init();				/* proxy, sql, ldap */
     get_mail_conf_str_table(str_table);
 
