@@ -255,8 +255,8 @@
 /*	Thus, the usual precautions need to be taken against malicious
 /*	inputs.
 /* DIAGNOSTICS
-/*	Problems are logged to \fBsyslogd\fR(8) or \fBpostlogd\fR(8),
-/*	and to the standard error stream.
+/*	Problems are logged to \fBsyslogd\fR(8) and to the standard error
+/*	stream.
 /* ENVIRONMENT
 /* .ad
 /* .fi
@@ -409,7 +409,6 @@
 /*	postdrop(1), mail posting utility
 /*	postfix(1), mail system control
 /*	postqueue(1), mail queue control
-/*	postlogd(8), Postfix logging
 /*	syslogd(8), system logging
 /* README_FILES
 /* .ad
@@ -447,6 +446,7 @@
 #include <stdlib.h>
 #include <signal.h>
 #include <fcntl.h>
+#include <syslog.h>
 #include <time.h>
 #include <errno.h>
 #include <ctype.h>
@@ -459,6 +459,7 @@
 #include <mymalloc.h>
 #include <vstream.h>
 #include <msg_vstream.h>
+#include <msg_syslog.h>
 #include <vstring_vstream.h>
 #include <username.h>
 #include <fullname.h>
@@ -472,7 +473,6 @@
 #include <name_code.h>
 #include <warn_stat.h>
 #include <clean_env.h>
-#include <maillog_client.h>
 
 /* Global library. */
 
@@ -1049,15 +1049,15 @@ int     main(int argc, char **argv)
 	debug_me = 1;
 
     /*
-     * Initialize. Set up logging. Read the global configuration file after
-     * command-line processing. Set up signal handlers so that we can clean
-     * up incomplete output.
+     * Initialize. Set up logging, read the global configuration file and
+     * extract configuration information. Set up signal handlers so that we
+     * can clean up incomplete output.
      */
     if ((slash = strrchr(argv[0], '/')) != 0 && slash[1])
 	argv[0] = slash + 1;
     msg_vstream_init(argv[0], VSTREAM_ERR);
     msg_cleanup(tempfail);
-    maillog_client_init(mail_task("sendmail"), MAILLOG_CLIENT_FLAG_NONE);
+    msg_syslog_init(mail_task("sendmail"), LOG_PID, LOG_FACILITY);
     set_mail_conf_str(VAR_PROCNAME, var_procname = mystrdup(argv[0]));
 
     /*
@@ -1112,7 +1112,7 @@ int     main(int argc, char **argv)
     update_env(import_env->argv);
     argv_free(import_env);
     /* Re-evaluate mail_task() after reading main.cf. */
-    maillog_client_init(mail_task("sendmail"), MAILLOG_CLIENT_FLAG_NONE);
+    msg_syslog_init(mail_task("sendmail"), LOG_PID, LOG_FACILITY);
     get_mail_conf_str_table(str_table);
 
     mail_dict_init();
