@@ -402,15 +402,6 @@ TLS_APPL_STATE *tls_client_init(const TLS_CLIENT_INIT_PROPS *props)
      */
     SSL_CTX_set_verify_depth(client_ctx, props->verifydepth + 1);
 
-    /*
-     * Presently we use TLS only with SMTP where truncation attacks are not
-     * possible as a result of application framing.  If we ever use TLS in
-     * some other application protocol where truncation could be relevant,
-     * we'd need to disable truncation detection conditionally, or explicitly
-     * clear the option in that code path.
-     */
-    off |= SSL_OP_IGNORE_UNEXPECTED_EOF;
-
     /*
      * Protocol selection is destination dependent, so we delay the protocol
      * selection options to the per-session SSL object.
@@ -418,11 +409,6 @@ TLS_APPL_STATE *tls_client_init(const TLS_CLIENT_INIT_PROPS *props)
     off |= tls_bug_bits();
     SSL_CTX_set_options(client_ctx, off);
 
-    /* Enable all supported protocols */
-#if OPENSSL_VERSION_NUMBER >= 0x1010000fUL
-    SSL_CTX_set_min_proto_version(client_ctx, 0);
-#endif
-
     /*
      * Set the call-back routine for verbose logging.
      */
@@ -921,24 +907,6 @@ TLS_SESS_STATE *tls_client_start(const TLS_CLIENT_START_PROPS *props)
     if (TLS_DANE_BASED(props->tls_level))
 	protomask |= TLS_PROTOCOL_SSLv2;
 
-    /*
-     * Allocate a new TLScontext for the new connection and get an SSL
-     * structure. Add the location of TLScontext to the SSL to later retrieve
-     * the information inside the tls_verify_certificate_callback().
-     * 
-     * If session caching was enabled when TLS was initialized, the cache type
-     * is stored in the client SSL context.
-     */
-    TLScontext = tls_alloc_sess_context(log_mask, props->namaddr);
-    TLScontext->cache_type = app_ctx->cache_type;
-
-    if ((TLScontext->con = SSL_new(app_ctx->ssl_ctx)) == NULL) {
-	msg_warn("Could not allocate 'TLScontext->con' with SSL_new()");
-	tls_print_errors();
-	tls_free_context(TLScontext);
-	return (0);
-    }
-
     /*
      * Per session cipher selection for sessions with mandatory encryption
      * 
@@ -946,11 +914,11 @@ TLS_SESS_STATE *tls_client_start(const TLS_CLIENT_START_PROPS *props)
      * to stay the same between connections, so we make use of a 1-element
      * cache to return the same result for identical inputs.
      */
-    cipher_list = tls_set_ciphers(TLScontext, props->cipher_grade,
+    cipher_list = tls_set_ciphers(app_ctx, "TLS", props->cipher_grade,
 				  props->cipher_exclusions);
     if (cipher_list == 0) {
-	/* already warned */
-	tls_free_context(TLScontext);
+	msg_warn("%s: %s: aborting TLS session",
+		 props->namaddr, vstring_str(app_ctx->why));
 	return (0);
     }
     if (log_mask & TLS_LOG_VERBOSE)
@@ -981,6 +949,17 @@ TLS_SESS_STATE *tls_client_start(const TLS_CLIENT_START_PROPS *props)
      */
     myserverid = tls_serverid_digest(props, protomask, cipher_list);
 
+    /*
+     * Allocate a new TLScontext for the new connection and get an SSL
+     * structure. Add the location of TLScontext to the SSL to later retrieve
+     * the information inside the tls_verify_certificate_callback().
+     * 
+     * If session caching was enabled when TLS was initialized, the cache type
+     * is stored in the client SSL context.
+     */
+    TLScontext = tls_alloc_sess_context(log_mask, props->namaddr);
+    TLScontext->cache_type = app_ctx->cache_type;
+
     TLScontext->serverid = myserverid;
     TLScontext->stream = props->stream;
     TLScontext->mdalg = props->mdalg;
@@ -988,6 +967,12 @@ TLS_SESS_STATE *tls_client_start(const TLS_CLIENT_START_PROPS *props)
     /* Alias DANE digest info from props */
     TLScontext->dane = props->dane;
 
+    if ((TLScontext->con = SSL_new(app_ctx->ssl_ctx)) == NULL) {
+	msg_warn("Could not allocate 'TLScontext->con' with SSL_new()");
+	tls_print_errors();
+	tls_free_context(TLScontext);
+	return (0);
+    }
     if (!SSL_set_ex_data(TLScontext->con, TLScontext_index, TLScontext)) {
 	msg_warn("Could not set application data for 'TLScontext->con'");
 	tls_print_errors();
@@ -1032,13 +1017,10 @@ TLS_SESS_STATE *tls_client_start(const TLS_CLIENT_START_PROPS *props)
 	 * avoid SNI, and there are no plans to support SNI in the Postfix
 	 * SMTP server).
 	 * 
-	 * Per RFC7672, the required SNI name is the TLSA "base domain" (the one
-	 * used to construct the "_25._tcp.<fqdn>" TLSA record DNS query).
-	 * 
 	 * Since the hostname is DNSSEC-validated, it must be a DNS FQDN and
 	 * thererefore valid for use with SNI.
 	 */
-	sni = props->dane->base_domain;
+	sni = props->host;
     } else if (props->sni && *props->sni) {
 	if (strcmp(props->sni, "hostname") == 0)
 	    sni = props->host;
@@ -1060,13 +1042,6 @@ TLS_SESS_STATE *tls_client_start(const TLS_CLIENT_START_PROPS *props)
 	    tls_free_context(TLScontext);
 	    return (0);
 	}
-	/*
-	 * The saved value is not presently used client-side, but could later
-	 * be logged if acked by the server (requires new client-side callback
-	 * to detect the ack).  For now this just maintains symmetry with the
-	 * server code, where do record the received SNI for logging.
-	 */
-	TLScontext->peer_sni = mystrdup(sni);
 	if (log_mask & TLS_LOG_DEBUG)
 	    msg_info("%s: SNI hostname: %s", props->namaddr, sni);
     }
