@@ -6,31 +6,18 @@
 /* SYNOPSIS
 /*	#include <tls_proxy.h>
 /*
-/*	int	tls_proxy_client_param_print(print_fn, stream, flags, ptr)
+/*	int     tls_proxy_client_init_print(print_fn, stream, flags, ptr)
 /*	ATTR_PRINT_MASTER_FN print_fn;
-/*	VSTREAM	*stream;
-/*	int	flags;
-/*	void	*ptr;
+/*	VSTREAM *stream;
+/*	int     flags;
+/*	void    *ptr;
 /*
-/*	int	tls_proxy_client_init_print(print_fn, stream, flags, ptr)
+/*	int     tls_proxy_client_start_print(print_fn, stream, flags, ptr)
 /*	ATTR_PRINT_MASTER_FN print_fn;
-/*	VSTREAM	*stream;
-/*	int	flags;
-/*	void	*ptr;
-/*
-/*	int	tls_proxy_client_start_print(print_fn, stream, flags, ptr)
-/*	ATTR_PRINT_MASTER_FN print_fn;
-/*	VSTREAM	*stream;
-/*	int	flags;
-/*	void	*ptr;
+/*	VSTREAM *stream;
+/*	int     flags;
+/*	void    *ptr;
 /* DESCRIPTION
-/*	tls_proxy_client_param_print() writes a TLS_CLIENT_PARAMS structure to
-/*	the named stream using the specified attribute print routine.
-/*	tls_proxy_client_param_print() is meant to be passed as a call-back to
-/*	attr_print(), thusly:
-/*
-/*	SEND_ATTR_FUNC(tls_proxy_client_param_print, (void *) param), ...
-/*
 /*	tls_proxy_client_init_print() writes a full TLS_CLIENT_INIT_PROPS
 /*	structure to the named stream using the specified attribute
 /*	print routine. tls_proxy_client_init_print() is meant to
@@ -70,10 +57,6 @@
 #include <attr.h>
 #include <msg.h>
 
-/* Global library. */
-
-#include <mail_params.h>
-
 /* TLS library. */
 
 #include <tls.h>
@@ -83,55 +66,6 @@
 #define STR(x) vstring_str(x)
 #define LEN(x) VSTRING_LEN(x)
 
-/* tls_proxy_client_param_print - send TLS_CLIENT_PARAMS over stream */
-
-int     tls_proxy_client_param_print(ATTR_PRINT_MASTER_FN print_fn, VSTREAM *fp,
-				             int flags, void *ptr)
-{
-    TLS_CLIENT_PARAMS *params = (TLS_CLIENT_PARAMS *) ptr;
-    int     ret;
-
-    if (msg_verbose)
-	msg_info("begin tls_proxy_client_param_print");
-
-    ret = print_fn(fp, flags | ATTR_FLAG_MORE,
-		   SEND_ATTR_STR(VAR_TLS_HIGH_CLIST, params->tls_high_clist),
-		   SEND_ATTR_STR(VAR_TLS_MEDIUM_CLIST,
-				 params->tls_medium_clist),
-		   SEND_ATTR_STR(VAR_TLS_LOW_CLIST, params->tls_low_clist),
-		   SEND_ATTR_STR(VAR_TLS_EXPORT_CLIST,
-				 params->tls_export_clist),
-		   SEND_ATTR_STR(VAR_TLS_NULL_CLIST, params->tls_null_clist),
-		   SEND_ATTR_STR(VAR_TLS_EECDH_AUTO, params->tls_eecdh_auto),
-		   SEND_ATTR_STR(VAR_TLS_EECDH_STRONG,
-				 params->tls_eecdh_strong),
-		   SEND_ATTR_STR(VAR_TLS_EECDH_ULTRA,
-				 params->tls_eecdh_ultra),
-		   SEND_ATTR_STR(VAR_TLS_BUG_TWEAKS, params->tls_bug_tweaks),
-		   SEND_ATTR_STR(VAR_TLS_SSL_OPTIONS,
-				 params->tls_ssl_options),
-		   SEND_ATTR_STR(VAR_TLS_DANE_DIGESTS,
-				 params->tls_dane_digests),
-		   SEND_ATTR_STR(VAR_TLS_MGR_SERVICE,
-				 params->tls_mgr_service),
-		   SEND_ATTR_STR(VAR_TLS_TKT_CIPHER, params->tls_tkt_cipher),
-		   SEND_ATTR_INT(VAR_TLS_DAEMON_RAND_BYTES,
-				 params->tls_daemon_rand_bytes),
-		   SEND_ATTR_INT(VAR_TLS_APPEND_DEF_CA,
-				 params->tls_append_def_CA),
-		   SEND_ATTR_INT(VAR_TLS_BC_PKEY_FPRINT,
-				 params->tls_bc_pkey_fprint),
-		   SEND_ATTR_INT(VAR_TLS_PREEMPT_CLIST,
-				 params->tls_preempt_clist),
-		   SEND_ATTR_INT(VAR_TLS_MULTI_WILDCARD,
-				 params->tls_multi_wildcard),
-		   ATTR_TYPE_END);
-    /* Do not flush the stream. */
-    if (msg_verbose)
-	msg_info("tls_proxy_client_param_print ret=%d", ret);
-    return (ret);
-}
-
 /* tls_proxy_client_init_print - send TLS_CLIENT_INIT_PROPS over stream */
 
 int     tls_proxy_client_init_print(ATTR_PRINT_MASTER_FN print_fn, VSTREAM *fp,
